using System;
using System.Runtime.Remoting.Messaging;

namespace KnownObjects
{
	/// <summary>
	/// Describes a callback called when a message is received.
	/// </summary>
	public interface IMessageReceiver
	{
		/// <summary>
		/// Is called by the server when a message is accepted.
		/// </summary>
		/// <param name="message">A message.</param>
		/// <param name="nickname">Nickname of the client who sent the message.</param>
		object ReceiveMessage(string message, string nickname);
	}

	/// <summary>
	/// Server chat room factory.
	/// </summary>
	public interface IChatServer
	{
		/// <summary>
		/// Performs log in to the chat room.
		/// </summary>
		/// <param name="nickname">Nickname.</param>
		/// <returns>Chat room interface.</returns>
		IChatRoom EnterToChatRoom();

		string GetTarget();
	}

	/// <summary>
	/// ChatRoom provides methods for the chatting.
	/// </summary>
	public interface IChatRoom
	{
		/// <summary>
		/// Sends the message to all clients.
		/// </summary>
		/// <param name="message">Message being sent.</param>
		string SendMessage(string message);
	}

	/// <summary>
	/// Describes functionality provided by a third-party server.
	/// </summary>
	public interface IThirdParty
	{
		/// <summary>
		/// Creates SSPI Server provider with the specified name.
		/// </summary>
		/// <param name="securityProviderName">The name of the Security Provider.</param>
		void CreateSecurityContext(string securityProviderName);

		/// <summary>
		/// Releases the established Security Sessions with the specified name.
		/// </summary>
		/// <param name="securityProviderName">The name of the Security Provider.</param>
		void ResetSecurityContext(string securityProviderName);

		/// <summary>
		/// Unregisters the specified Security Provider.
		/// </summary>
		/// <param name="securityProviderName">The name of the Security Provider.</param>
		void ReleaseSecurityContext(string securityProviderName);

		/// <summary>
		/// Returns the SSPI target.
		/// </summary>
		/// <returns></returns>
		string GetTarget();

		/// <summary>
		/// To check delegation.
		/// </summary>
		/// <param name="message"></param>
		void MessageHasBeenSent(string message);
	}
}
